#
# Cookbook:: desktop-config
# Resource:: macos_desktop_screensaver
#
# Copyright:: Chef Software, Inc.
#

provides :macos_desktop_screensaver
provides :desktop_screensaver, platform: 'mac_os_x'

description 'Use the **macos_desktop_screensaver** resource to configure secure screensaver settings on macOS systems.'
introduced '1.0'
examples <<~DOC
  **Turn on the Screensaver**:

  ```ruby
  desktop_screensaver 'Sets up the screensaver to come on after 30 minutes of idle time and require a password' do
    idle_time 30
    require_password true
    delay_before_password_prompt 5
    action :set
  end
  ```
DOC
unified_mode true

property :idle_time, Integer, equal_to: [0, 1, 2, 5, 10, 20, 30, 60], default: 20, description: 'Time in minutes before the the Screensaver comes on. Must be one of these values: 1, 2, 5, 10, 20, 30, 60'

EXCLUDED_USERS = %W(daemon nobody root).freeze

def all_users
  result = []
  Etc.passwd { |user| result << user.name }
  result.grep(/^[a-zA-Z]+/) - EXCLUDED_USERS
end

load_current_value do
  idle_time screensaver_value('idleTime') / 60
end

def screensaver_value(setting)
  current_user = Etc.getlogin
  cmd = "defaults -currentHost read com.apple.screensaver '#{setting}'"
  shell_out(cmd, user: current_user).stdout.to_i
end

action :set do
  all_users.each do |user_name|
    converge_if_changed :idle_time do
      macos_userdefaults 'update the screensaver idleTime' do
        domain 'com.apple.screensaver'
        host :current
        key 'idleTime'
        value "#{new_resource.idle_time * 60}"
        type 'int'
        user user_name
        action :write
      end
    end
  end
end
