#
# Cookbook:: desktop-config
# Resource:: macos_power_management
#
# Copyright:: Chef Software, Inc.
#

provides :macos_power_management

description 'Use the **macos_power_management** resource to set the power settings of a kiosk-style device when you need it always-on'
introduced '1.0'
examples <<~DOC
  **Configure Power Management settings**:

  ```ruby
  macos_power_management 'Set the Device to a defined power level' do
    computer_sleep_time 'never'
    display_sleep_time 'never'
    disk_sleep_time 'never'
    action :set
  end
  ```
DOC

property :computer_sleep_time, String, default: 'never', description: 'A time value between 1-60 minutes or "never" to use to set the computer to sleep after. Defaults to never'
property :display_sleep_time, String, default: 'never', description: 'A time value between 1-60 minutes or "never" to use to set the display to sleep after. Defaults to never'
property :disk_sleep_time, String, default: 'never', description: 'A time value between 1-60 minutes or "never" to use to set the hard disk to sleep after. Defaults to never'

load_current_value do
  computer_sleep_time timer_value('getcomputersleep')
  display_sleep_time timer_value('getdisplaysleep')
  disk_sleep_time timer_value('getharddisksleep')
end

def timer_value(setting)
  cmd = "/usr/sbin/systemsetup -#{setting}"
  output = shell_out(cmd).stdout
  case output
  when /never/i
    'never'
  when /: after (\d+)/
    Regexp.last_match(1)
  else
    raise "'#{cmd}' returned unrecognized output: '#{output}'"
  end
end

action :set do
  converge_if_changed :computer_sleep_time do
    execute "set computer to sleep - #{new_resource.computer_sleep_time}" do
      command "/usr/sbin/systemsetup -setcomputersleep #{new_resource.computer_sleep_time}"
      user 'root'
    end
  end

  converge_if_changed :display_sleep_time do
    execute "set display to sleep - #{new_resource.display_sleep_time}" do
      command "/usr/sbin/systemsetup -setdisplaysleep #{new_resource.display_sleep_time}"
      user 'root'
    end
  end

  converge_if_changed :disk_sleep_time do
    execute "set disk to sleep - #{new_resource.disk_sleep_time}" do
      command "/usr/sbin/systemsetup -setharddisksleep #{new_resource.disk_sleep_time}"
      user 'root'
    end
  end
end
