#
# Cookbook:: desktop-config
# Resource:: windows_desktop_screensaver
#
# Copyright:: Chef Software, Inc.
#

provides :windows_desktop_screensaver
provides :desktop_screensaver, platform: 'windows'

description 'Use the **windows_desktop_screensaver** resource to configure secure screensaver settings on Windows systems.'
introduced '1.0'
examples <<~DOC
  **Secure the desktop with a screensaver and password**:

  ```ruby
  desktop_screensaver 'Sets up a Screensaver to come on and require a password after xx minutes' do
    require_password true
    idle_time 20
    allow_lower_user_idle_time false
    screensaver_name 'mystify.scr'
    action :enable
  end
  ```

  **Disable requiring a screensaver with a password**:

  ```ruby
  desktop_screensaver 'Disable the screensaver' do
    action :disable
  end
  ```
DOC
unified_mode true

property :screensaver_name, String, description: 'The name of a specific or custom screensaver to enable.'
property :require_password, [true, false], default: true, description: 'Require a password when waking from the screensaver.'
property :idle_time, Integer, default: 20, description: 'The amount of idle time in minutes before the screensaver comes on.'
property :allow_lower_user_idle_time, [true, false], default: false, description: 'Allow users to set their screen saver idle time lower than the system requirements.'

action :enable do
  require 'wmi-lite/wmi' unless defined?(WmiLite::Wmi)
  wmi = WmiLite::Wmi.new
  user_profiles = wmi.instances_of('Win32_UserProfile')

  user_profiles.each do |profile|
    sid = profile['sid']
    next unless registry_key_exists?("HKEY_USERS\\#{sid}")

    # Configurations in Software\Policies prevents user modification
    hkey_software_policies = "HKEY_USERS\\#{sid}\\Software\\Policies\\Microsoft\\Windows\\Control Panel\\Desktop"
    # Allows for user-tunable options
    hkey_user_control_panel = "HKEY_USERS\\#{sid}\\Control Panel\\Desktop"

    timeout = new_resource.idle_time * 60

    # Use 'screensaver_name' if it is set, otherwise do not change user's screensaver from current
    current_scrnsave_name = screensaver_selected?(hkey_user_control_panel)
    screensaver_name = new_resource.screensaver_name && !new_resource.screensaver_name.empty? ? new_resource.screensaver_name : current_scrnsave_name

    if new_resource.allow_lower_user_idle_time == true
      current_timeout = if registry_value_exists?(hkey_user_control_panel, { name: 'ScreenSaveTimeOut' })
                          (registry_get_values(hkey_user_control_panel).find { |value| value[:name] == 'ScreenSaveTimeOut' })[:data].to_i
                        else
                          timeout
                        end

      # If the current_timeout value is 0/blank/nil or greater than the configured timeout, use the configured timeout.
      timeout = (current_timeout < 1 || current_timeout > timeout) ? timeout.to_s : current_timeout.to_s

      if registry_value_exists?(hkey_software_policies, { name: 'ScreenSaveTimeOut' })
        registry_key hkey_software_policies do
          recursive true
          values [{
            name: 'ScreenSaveTimeOut',
            }]
          action :delete
        end
      end

      registry_key hkey_user_control_panel do
        recursive true
        values [{
          name: 'ScreenSaveTimeOut',
          type: :string,
          data: timeout.to_s,
        }]
        action :create
      end

    else
      registry_key hkey_software_policies do
        recursive true
        values [{
          name: 'ScreenSaveTimeOut',
          type: :string,
          data: timeout.to_s,
        }]
        action :create
      end
    end # end allow_lower_user_idle_time

    registry_key hkey_software_policies do
      recursive true
      values [
        {
          name: 'ScreenSaveActive',
          type: :dword,
          data: 1,
        },
        {
          name: 'ScreenSaverIsSecure',
          type: :dword,
          data: new_resource.require_password ? 1 : 0,
        }]
      action :create
    end
    registry_key hkey_user_control_panel do
      values [{
        name: 'scrnsave.exe',
        type: :string,
        data: screensaver_name,
      }]
      action :create
    end
  end
end

action :disable do
  registry_key hkey_software_policies do
    recursive true
    values [{
      name: 'ScreenSaveActive',
      type: :dword,
      data: 0,
      }]
  end
end

action_class do
  def screensaver_selected?(path)
    current_scrnsave = if registry_value_exists?("#{path}", { name: 'scrnsave.exe' })
                         (registry_get_values("#{path}").find { |value| value[:name].downcase == 'scrnsave.exe' })[:data].to_s
                       end
    # A screensaver needs to be set for enable functionality to work properly.  If none is chosen currently then the Blank 'scrnsave.scr' will be chosen.
    if current_scrnsave && !current_scrnsave.empty?
      current_scrnsave
    else
      'scrnsave.scr'
    end
  end
end
