#
# Cookbook:: desktop-config
# Resource:: windows_power_management
#
# Copyright:: Chef Software, Inc.
#

provides :windows_power_management

description 'Use the **windows_power_management** resource to set the power settings of a kiosk-style device when you need it always-on'
introduced '1.0'
examples <<~DOC
  **Setup a Power Management Policy**:

  ```ruby
  windows_power_management 'Set the Device to a defined power level' do
    power_scheme_label 'Unrestricted'
    power_level 'ultimate'
    monitor_timeout 15
    disk_timeout 0
    standby_timeout 0
    hibernate_timeout 0
    action :set
  end
  ```
DOC
unified_mode true

property :power_scheme_label, String, description: 'A label name to prefix your power scheme with. The code duplicates the existing power scheme to keep it distinct'
property :power_level, String, default: 'balanced', description: 'There are 2 levels of power - balanced, and ultimate.'
property :monitor_timeout, Integer, description: 'The amount of time in minutes to wait before turning off the display'
property :disk_timeout, Integer, description: 'The amount of time in minutes to wait before turning off the HD'
property :standby_timeout, Integer, description: 'The amount of time in minutes to wait before putting the system into standby'
property :hibernate_timeout, Integer, description: 'The amount of time in minutes to wait before hibernating the system'

load_current_value do
  values = powershell_exec(<<-CODE).result
    $MyPowerPlan = powercfg /GetActiveScheme
    $PowerPlanGuid = $MyPowerPlan.Split(' ')[3]
    $DisplaySettings = powercfg /QUERY $PowerPlanGuid '7516b95f-f776-4464-8c53-06167f40cc99' '3c0bc021-c8a8-4e07-a973-6b14cbcb2b7e'
    [int64]$DisplayTime = ($DisplaySettings[9] -split ("\s+"))[6]
    $DisplayTime / 60

    $HDSettings = powercfg /QUERY $PowerPlanGuid '0012ee47-9041-4b5d-9b77-535fba8b1442' '6738e2c4-e8a5-4a42-b16a-e040e769756e'
    [int64]$HDTime = ($HDSettings[10] -split ("\s+"))[6]
    $HDTime / 60

    $SleepSettings = powercfg /QUERY $PowerPlanGuid '238c9fa8-0aad-41ed-83f4-97be242c8f20' '29f6c1db-86da-48c5-9fdb-f2b67b1f44da'
    [int64]$SleepTime = ($SleepSettings[9] -split ("\s+"))[6]
    $SleepTime / 60

    $HibernateSettings = powercfg /QUERY $PowerPlanGuid '238c9fa8-0aad-41ed-83f4-97be242c8f20' '9d7815a6-7ee4-497e-8888-515a05f02364'
    [int64]$HibernateTime = ($HibernateSettings[10] -split ("\s+"))[6]
    $HibernateTime / 60
  CODE

  monitor_timeout values[0]
  disk_timeout values[1]
  standby_timeout values[2]
  hibernate_timeout values[3]
end

action :set do
  if new_resource.power_level == 'balanced'
    converge_if_changed :monitor_timeout, :disk_timeout, :standby_timeout, :hibernate_timeout do
      powershell_script 'setting balanced power scheme' do
        code <<-CODE
          $dup = powercfg -duplicatescheme 381b4222-f694-41f0-9685-ff5bb260df2e
          $guid = $dup.Split(' ')[3]
          powercfg /changename "$($guid)" "#{new_resource.power_scheme_label} Balanced"
          powercfg /setactive "$($guid)"
          powercfg /change monitor-timeout-ac #{new_resource.monitor_timeout}
          powercfg /change disk-timeout-ac #{new_resource.disk_timeout}
          powercfg /change standby-timeout-ac #{new_resource.standby_timeout}
          powercfg /change hibernate-timeout-ac #{new_resource.hibernate_timeout}
        CODE
      end
    end
  elsif new_resource.power_level == 'ultimate'
    converge_if_changed :monitor_timeout, :disk_timeout, :standby_timeout, :hibernate_timeout do
      powershell_script 'setting ultimate power scheme' do
        code <<-CODE
          $dup = powercfg -duplicatescheme e9a42b02-d5df-448d-aa00-03f14749eb61
          $guid = $dup.Split(' ')[3]
          powercfg /changename "$($guid)" "#{new_resource.power_scheme_label} Ultimate Power"
          powercfg /setactive "$($guid)"
          powercfg /change monitor-timeout-ac #{new_resource.monitor_timeout}
          powercfg /change disk-timeout-ac #{new_resource.disk_timeout}
          powercfg /change standby-timeout-ac #{new_resource.standby_timeout}
          powercfg /change hibernate-timeout-ac #{new_resource.hibernate_timeout}
        CODE
      end
    end
  end
end
